'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

function hasFilters(search, filters) {
  if (search === void 0) {
    search = {
      query: '',
      fields: []
    };
  }

  if (filters === void 0) {
    filters = {};
  }

  if (search && search.query && search.query.trim()) return true;
  return Object.keys(filters).some(function (item) {
    return filters[item] && filters[item].length;
  });
}

function highlightSearch(value, searchQuery) {
  var reg = new RegExp(searchQuery.replace(/[|\\{}()[\]^$+*?.]/g, '\\$&'), 'i');
  return value.replace(reg, function (str) {
    return "<mark class=\"use-filter-highlight\">" + str + "</mark>";
  });
}
function filterFn(_ref) {
  var _ref$data = _ref.data,
      data = _ref$data === void 0 ? [] : _ref$data,
      _ref$filters = _ref.filters,
      filters = _ref$filters === void 0 ? {} : _ref$filters,
      _ref$search = _ref.search,
      search = _ref$search === void 0 ? {
    query: '',
    fields: []
  } : _ref$search;
  var result = Object.assign(data);

  if (search && search.query && search.query.trim()) {
    var queryLowerCase = search.query.toLowerCase();

    if (search.enableHighlighting) {
      result = data.reduce(function (acc, item) {
        var hasSearchTerm = false;
        var itemToReturn = Object.assign({}, item);
        search.fields.forEach(function (field) {
          var fieldVal = item[field].toString().toLowerCase();

          if (fieldVal.includes(queryLowerCase)) {
            hasSearchTerm = true; // @ts-ignore

            itemToReturn[field] = highlightSearch(fieldVal, queryLowerCase);
          }
        });

        if (hasSearchTerm) {
          return acc.concat(itemToReturn);
        }

        return acc;
      }, []);
    } else {
      result = data.filter(function (item) {
        return search.fields.some(function (field) {
          var fieldLowerCase = item[field].toLowerCase();
          return fieldLowerCase.includes(queryLowerCase);
        });
      });
    }
  }

  if (filters && Object.keys(filters).length) {
    Object.keys(filters).forEach(function (field) {
      if (filters[field] && filters[field].length) {
        result = result.filter(function (item) {
          return filters[field].includes(item[field]);
        });
      }
    });
  }

  return result;
}

/* eslint-disable react-hooks/exhaustive-deps */
var code = "\n  " + /*#__PURE__*/highlightSearch.toString() + "\n  " + /*#__PURE__*/filterFn.toString() + "\n  onmessage = function(e) {\n    const params = e.data;\n    const filterData = filterFn(params)\n    postMessage(filterData);\n  };\n";
var worker = /*#__PURE__*/new Worker( /*#__PURE__*/URL.createObjectURL( /*#__PURE__*/new Blob([code], {
  type: 'text/javascript'
})));
function useFilter(_ref) {
  var data = _ref.data,
      search = _ref.search,
      filters = _ref.filters;

  var _useState = react.useState(false),
      loading = _useState[0],
      setLoading = _useState[1];

  var _useState2 = react.useState([]),
      result = _useState2[0],
      setResult = _useState2[1];

  var isHavingFilters = react.useMemo(function () {
    return hasFilters(search, filters);
  }, [search, filters]);
  react.useEffect(function () {
    var isMounted = true;

    if (isMounted) {
      setLoading(true);
    }

    if (isHavingFilters) {
      worker.postMessage({
        data: data,
        search: search,
        filters: filters
      });

      worker.onmessage = function (e) {
        if (isMounted) {
          setLoading(false);
          setResult(e.data);
        }
      };

      worker.onerror = function (e) {
        console.error("Web worker error", e);

        if (isMounted) {
          setLoading(false);
        }
      };
    }

    return function () {
      isMounted = false;
    };
  }, [search, filters]);
  return {
    loading: loading,
    data: isHavingFilters ? result : data
  };
}

exports.useFilter = useFilter;
//# sourceMappingURL=usefilter.cjs.development.js.map
